package sun.applet;

import java.security.PrivilegedAction;
import java.util.NoSuchElementException;
import java.util.Enumeration;
import java.io.InputStream;
import java.net.URLConnection;
import sun.misc.IOUtils;
import java.io.BufferedInputStream;
import java.net.HttpURLConnection;
import java.security.Permission;
import java.io.File;
import java.io.FilePermission;
import java.security.PermissionCollection;
import java.security.PrivilegedActionException;
import java.security.PrivilegedExceptionAction;
import sun.net.www.ParseUtil;
import java.io.IOException;
import java.net.MalformedURLException;
import java.security.AccessController;
import java.security.cert.Certificate;
import java.util.HashMap;
import sun.awt.AppContext;
import java.security.AccessControlContext;
import java.security.CodeSource;
import java.net.URL;
import java.net.URLClassLoader;

public class AppletClassLoader extends URLClassLoader
{
    private URL base;
    private CodeSource codesource;
    private AccessControlContext acc;
    private boolean exceptionStatus;
    private final Object threadGroupSynchronizer;
    private final Object grabReleaseSynchronizer;
    private boolean codebaseLookup;
    private volatile boolean allowRecursiveDirectoryRead;
    private Object syncResourceAsStream;
    private Object syncResourceAsStreamFromJar;
    private boolean resourceAsStreamInCall;
    private boolean resourceAsStreamFromJarInCall;
    private AppletThreadGroup threadGroup;
    private AppContext appContext;
    int usageCount;
    private HashMap jdk11AppletInfo;
    private HashMap jdk12AppletInfo;
    private static AppletMessageHandler mh;
    
    protected AppletClassLoader(URL url) {
        super(new URL[0]);
        this.exceptionStatus = false;
        this.threadGroupSynchronizer = new Object();
        this.grabReleaseSynchronizer = new Object();
        this.codebaseLookup = true;
        this.allowRecursiveDirectoryRead = true;
        this.syncResourceAsStream = new Object();
        this.syncResourceAsStreamFromJar = new Object();
        this.resourceAsStreamInCall = false;
        this.resourceAsStreamFromJarInCall = false;
        this.usageCount = 0;
        this.jdk11AppletInfo = new HashMap();
        this.jdk12AppletInfo = new HashMap();
        this.base = url;
        this.codesource = new CodeSource(url, (Certificate[])null);
        this.acc = AccessController.getContext();
    }
    
    public void disableRecursiveDirectoryRead() {
        this.allowRecursiveDirectoryRead = false;
    }
    
    void setCodebaseLookup(boolean codebaseLookup) {
        this.codebaseLookup = codebaseLookup;
    }
    
    URL getBaseURL() {
        return this.base;
    }
    
    @Override
    public URL[] getURLs() {
        URL[] urLs = super.getURLs();
        URL[] array = new URL[urLs.length + 1];
        System.arraycopy(urLs, 0, array, 0, urLs.length);
        array[array.length - 1] = this.base;
        return array;
    }
    
    protected void addJar(String spec) throws IOException {
        URL url;
        try {
            url = new URL(this.base, spec);
        }
        catch (MalformedURLException ex) {
            throw new IllegalArgumentException("name");
        }
        this.addURL(url);
    }
    
    public synchronized Class loadClass(String name, boolean resolve) throws ClassNotFoundException {
        int lastIndex = name.lastIndexOf(46);
        if (lastIndex != -1) {
            SecurityManager securityManager = System.getSecurityManager();
            if (securityManager != null) {
                securityManager.checkPackageAccess(name.substring(0, lastIndex));
            }
        }
        try {
            return super.loadClass(name, resolve);
        }
        catch (ClassNotFoundException ex) {
            throw ex;
        }
        catch (RuntimeException ex2) {
            throw ex2;
        }
        catch (Error error) {
            throw error;
        }
    }
    
    @Override
    protected Class findClass(String substring) throws ClassNotFoundException {
        int index = substring.indexOf(";");
        String substring2 = "";
        if (index != -1) {
            substring2 = substring.substring(index, substring.length());
            substring = substring.substring(0, index);
        }
        try {
            return super.findClass(substring);
        }
        catch (ClassNotFoundException ex) {
            if (!this.codebaseLookup) {
                throw new ClassNotFoundException(substring);
            }
            final String string = new StringBuffer(ParseUtil.encodePath(substring.replace('.', '/'), false)).append(".class").append(substring2).toString();
            try {
                byte[] b = (byte[])(byte[])AccessController.doPrivileged(new PrivilegedExceptionAction() {
                    @Override
                    public Object run() throws IOException {
                        try {
                            URL url = new URL(AppletClassLoader.this.base, string);
                            if (AppletClassLoader.this.base.getProtocol().equals(url.getProtocol()) && AppletClassLoader.this.base.getHost().equals(url.getHost()) && AppletClassLoader.this.base.getPort() == url.getPort()) {
                                return AppletClassLoader.access$100(url);
                            }
                            return null;
                        }
                        catch (Exception ex) {
                            return null;
                        }
                    }
                }, this.acc);
                if (b != null) {
                    return this.defineClass(substring, b, 0, b.length, this.codesource);
                }
                throw new ClassNotFoundException(substring);
            }
            catch (PrivilegedActionException ex2) {
                throw new ClassNotFoundException(substring, ex2.getException());
            }
        }
    }
    
    @Override
    protected PermissionCollection getPermissions(CodeSource codesource) {
        PermissionCollection permissions = super.getPermissions(codesource);
        URL location = codesource.getLocation();
        String pathname = null;
        Permission permission;
        try {
            permission = location.openConnection().getPermission();
        }
        catch (IOException ex) {
            permission = null;
        }
        if ((permission instanceof FilePermission)) {
            pathname = permission.getName();
        }
        else if (permission == null && location.getProtocol().equals("file")) {
            pathname = ParseUtil.decode(location.getFile().replace('/', File.separatorChar));
        }
        if (pathname != null) {
            if (!pathname.endsWith(File.separator)) {
                int lastIndex = pathname.lastIndexOf((int)File.separatorChar);
                if (lastIndex != -1) {
                    permissions.add(new FilePermission(pathname.substring(0, lastIndex + 1) + "-", "read"));
                }
            }
            int directory = new File(pathname).isDirectory();
            if (this.allowRecursiveDirectoryRead) {
                if (directory == 0 && !pathname.toLowerCase().endsWith(".jar") && !pathname.toLowerCase().endsWith(".zip")) {
                    return permissions;
                }
                Permission permission2;
                try {
                    permission2 = this.base.openConnection().getPermission();
                }
                catch (IOException ex2) {
                    permission2 = null;
                }
                if ((permission2 instanceof FilePermission)) {
                    String s2 = permission2.getName();
                    if (s2.endsWith(File.separator)) {
                        s2 += "-";
                    }
                    permissions.add(new FilePermission(s2, "read"));
                }
                else if (permission2 == null && this.base.getProtocol().equals("file")) {
                    String s = ParseUtil.decode(this.base.getFile().replace('/', File.separatorChar));
                    if (s.endsWith(File.separator)) {
                        s += "-";
                    }
                    permissions.add(new FilePermission(s, "read"));
                }
            }
        }
        return permissions;
    }
    
    private static byte[] getBytes(URL url) throws IOException {
        URLConnection openConnection = url.openConnection();
        if (openConnection instanceof HttpURLConnection && ((HttpURLConnection)openConnection).getResponseCode() >= 400) {
            throw new IOException("open HTTP connection failed.");
        }
        int contentLength = openConnection.getContentLength();
        if (contentLength == -1) {
            contentLength = Integer.MAX_VALUE;
        }
        BufferedInputStream o = new BufferedInputStream(openConnection.getInputStream());
        byte[] nBytesOrEOF;
        try {
            nBytesOrEOF = IOUtils.readNBytesOrEOF(o, contentLength);
        }
        finally {
            o.close();
        }
        return nBytesOrEOF;
    }
    
    @Override
    public InputStream getResourceAsStream(String s) {
        if (s == null) {
            throw new NullPointerException("name");
        }
        try {
            InputStream inputStream = null;
            synchronized (this.syncResourceAsStream) {
                this.resourceAsStreamInCall = true;
                inputStream = super.getResourceAsStream(s);
                this.resourceAsStreamInCall = false;
            }
            if (this.codebaseLookup && inputStream == null) {
                inputStream = new URL(this.base, ParseUtil.encodePath(s, false)).openStream();
                return inputStream;
            }
            return inputStream;
        }
        catch (Exception ex) {
            return null;
        }
    }
    
    public InputStream getResourceAsStreamFromJar(String name) {
        if (name == null) {
            throw new NullPointerException("name");
        }
        try {
            InputStream resourceAsStream = null;
            synchronized (this.syncResourceAsStreamFromJar) {
                this.resourceAsStreamFromJarInCall = true;
                resourceAsStream = super.getResourceAsStream(name);
                this.resourceAsStreamFromJarInCall = false;
            }
            return resourceAsStream;
        }
        catch (Exception ex) {
            return null;
        }
    }
    
    @Override
    public URL findResource(String s) {
        URL resource = super.findResource(s);
        if (s.startsWith("META-INF/")) {
            return resource;
        }
        if (!this.codebaseLookup) {
            return resource;
        }
        if (resource == null) {
            boolean resourceAsStreamFromJarInCall = false;
            synchronized (this.syncResourceAsStreamFromJar) {
                resourceAsStreamFromJarInCall = this.resourceAsStreamFromJarInCall;
            }
            if (!(resourceAsStreamFromJarInCall == 0)) {
                return null;
            }
            boolean resourceAsStreamInCall = false;
            synchronized (this.syncResourceAsStream) {
                resourceAsStreamInCall = this.resourceAsStreamInCall;
            }
            if (resourceAsStreamInCall == 0) {
                try {
                    resource = new URL(this.base, ParseUtil.encodePath(s, false));
                    if (!this.resourceExists(resource)) {
                        resource = null;
                    }
                }
                catch (Exception ex) {
                    resource = null;
                }
            }
        }
        return resource;
    }
    
    private boolean resourceExists(URL url) {
        boolean n = true;
        try {
            URLConnection openConnection = url.openConnection();
            if ((openConnection instanceof HttpURLConnection)) {
                HttpURLConnection httpURLConnection = (HttpURLConnection)openConnection;
                httpURLConnection.setRequestMethod("HEAD");
                int responseCode = httpURLConnection.getResponseCode();
                if (responseCode == 200) {
                    return true;
                }
                if (responseCode >= 400) {
                    return false;
                }
            }
            else {
                openConnection.getInputStream().close();
            }
        }
        catch (Exception ex) {
            n = false;
        }
        return n;
    }
    
    @Override
    public Enumeration findResources(String s) throws IOException {
        final Enumeration<URL> resources = super.findResources(s);
        if (s.startsWith("META-INF/")) {
            return resources;
        }
        if (!this.codebaseLookup) {
            return resources;
        }
        final URL url = new URL(this.base, ParseUtil.encodePath(s, false));
        if (!this.resourceExists(url)) {
            url = null;
        }
        return new Enumeration() {
            private boolean done;
            
            @Override
            public Object nextElement() {
                if (!this.done) {
                    if (resources.hasMoreElements()) {
                        return resources.nextElement();
                    }
                    this.done = true;
                    if (url != null) {
                        return url;
                    }
                }
                throw new NoSuchElementException();
            }
            
            @Override
            public boolean hasMoreElements() {
                return !this.done && (resources.hasMoreElements() || url != null);
            }
        };
    }
    
    Class loadCode(String s) throws ClassNotFoundException {
        s = s.replace('/', '.');
        s = s.replace(File.separatorChar, '.');
        String substring = null;
        int index = s.indexOf(";");
        if (index != -1) {
            substring = s.substring(index, s.length());
            s = s.substring(0, index);
        }
        if (s.endsWith(".class") || s.endsWith(".java")) {
            s = s.substring(0, s.lastIndexOf(46));
        }
        try {
            if (substring != null) {
                s = new StringBuffer(s).append(substring).toString();
            }
            return this.loadClass(s);
        }
        catch (ClassNotFoundException ex) {
            if (substring != null) {
                s = new StringBuffer(s).append(substring).toString();
            }
            return this.loadClass(s);
        }
    }
    
    public ThreadGroup getThreadGroup() {
        synchronized (this.threadGroupSynchronizer) {
            if (this.threadGroup == null || this.threadGroup.isDestroyed()) {
                AccessController.doPrivileged(new PrivilegedAction() {
                    @Override
                    public Object run() {
                        AppletClassLoader.this.threadGroup = new AppletThreadGroup(AppletClassLoader.this.base + "-threadGroup");
                        AppContextCreator appContextCreator = new AppContextCreator(AppletClassLoader.this.threadGroup);
                        appContextCreator.setContextClassLoader(AppletClassLoader.this);
                        appContextCreator.start();
                        try {
                            synchronized (appContextCreator.syncObject) {
                                while (!appContextCreator.created) {
                                    appContextCreator.syncObject.wait();
                                }
                            }
                        }
                        catch (InterruptedException ex) {}
                        AppletClassLoader.this.appContext = appContextCreator.appContext;
                        return null;
                    }
                });
            }
            return this.threadGroup;
        }
    }
    
    public AppContext getAppContext() {
        return this.appContext;
    }
    
    public void grab() {
        synchronized (this.grabReleaseSynchronizer) {
            ++this.usageCount;
        }
        this.getThreadGroup();
    }
    
    protected void setExceptionStatus() {
        this.exceptionStatus = true;
    }
    
    public boolean getExceptionStatus() {
        return this.exceptionStatus;
    }
    
    protected void release() {
        AppContext resetAppContext = null;
        synchronized (this.grabReleaseSynchronizer) {
            if (this.usageCount > 1) {
                --this.usageCount;
            }
            else {
                synchronized (this.threadGroupSynchronizer) {
                    resetAppContext = this.resetAppContext();
                }
            }
        }
        if (resetAppContext != null) {
            try {
                resetAppContext.dispose();
            }
            catch (IllegalThreadStateException ex) {}
        }
    }
    
    protected AppContext resetAppContext() {
        AppContext appContext = null;
        synchronized (this.threadGroupSynchronizer) {
            appContext = this.appContext;
            this.usageCount = 0;
            this.appContext = null;
            this.threadGroup = null;
        }
        return appContext;
    }
    
    void setJDK11Target(Class clazz, boolean b) {
        this.jdk11AppletInfo.put(clazz.toString(), Boolean.valueOf(b));
    }
    
    void setJDK12Target(Class clazz, boolean b) {
        this.jdk12AppletInfo.put(clazz.toString(), Boolean.valueOf(b));
    }
    
    Boolean isJDK11Target(Class clazz) {
        return (Boolean)this.jdk11AppletInfo.get(clazz.toString());
    }
    
    Boolean isJDK12Target(Class clazz) {
        return (Boolean)this.jdk12AppletInfo.get(clazz.toString());
    }
    
    private static void printError(String s, Throwable t) {
        String x = null;
        if (t == null) {
            x = AppletClassLoader.mh.getMessage("filenotfound", s);
        }
        else if ((t instanceof IOException)) {
            x = AppletClassLoader.mh.getMessage("fileioexception", s);
        }
        else if ((t instanceof ClassFormatError)) {
            x = AppletClassLoader.mh.getMessage("fileformat", s);
        }
        else if ((t instanceof ThreadDeath)) {
            x = AppletClassLoader.mh.getMessage("filedeath", s);
        }
        else if ((t instanceof Error)) {
            x = AppletClassLoader.mh.getMessage("fileerror", t.toString(), s);
        }
        if (x != null) {
            System.err.println(x);
        }
    }
    
    static byte[] access$100(URL url) throws IOException {
        return getBytes(url);
    }
    
    static {
        AppletClassLoader.mh = new AppletMessageHandler("appletclassloader");
    }
}
